package com.schoolinout.system.service.impl;

import com.schoolinout.common.exception.ServiceException;
import com.schoolinout.common.utils.DateUtils;
import com.schoolinout.common.utils.bean.BeanUtils;
import com.schoolinout.system.domain.Clazz;
import com.schoolinout.system.domain.Teacher;
import com.schoolinout.system.domain.TeacherClassRelation;
import com.schoolinout.system.domain.dto.AddTeacherClassRelationDto;
import com.schoolinout.system.domain.dto.BindClassTeacherMain;
import com.schoolinout.system.domain.dto.InsertTeacherClassRelationDto;
import com.schoolinout.system.domain.vo.TeacherClassInfoVo;
import com.schoolinout.system.mapper.ClazzMapper;
import com.schoolinout.system.mapper.TeacherClassRelationMapper;
import com.schoolinout.system.mapper.TeacherMapper;
import com.schoolinout.system.service.ITeacherClassRelationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 老师-班级关系Service业务层处理
 *
 * @author sakura
 * @date 2024-01-04
 */
@Service
public class TeacherClassRelationServiceImpl implements ITeacherClassRelationService {
    @Autowired
    private TeacherClassRelationMapper teacherClassRelationMapper;
    @Autowired
    private TeacherMapper teacherMapper;
    @Autowired
    private ClazzMapper clazzMapper;

    /**
     * 查询老师-班级关系
     *
     * @param id 老师-班级关系主键
     * @return 老师-班级关系
     */
    @Override
    public TeacherClassRelation selectTeacherClassRelationById(Long id) {
        return teacherClassRelationMapper.selectTeacherClassRelationById(id);
    }

    /**
     * 查询老师-班级关系列表
     *
     * @param teacherClassRelation 老师-班级关系
     * @return 老师-班级关系
     */
    @Override
    public List<TeacherClassRelation> selectTeacherClassRelationList(TeacherClassRelation teacherClassRelation) {
        return teacherClassRelationMapper.selectTeacherClassRelationList(teacherClassRelation);
    }

    /**
     * 新增老师-班级关系
     *
     * @param teacherClassRelation 老师-班级关系
     * @return 结果
     */
    @Override
    public int insertTeacherClassRelation(TeacherClassRelation teacherClassRelation) {
        teacherClassRelation.setCreateTime(DateUtils.getNowDate());
        return teacherClassRelationMapper.insertTeacherClassRelation(teacherClassRelation);
    }

    /**
     * 修改老师-班级关系
     *
     * @param teacherClassRelation 老师-班级关系
     * @return 结果
     */
    @Override
    public int updateTeacherClassRelation(TeacherClassRelation teacherClassRelation) {
        teacherClassRelation.setUpdateTime(DateUtils.getNowDate());
        return teacherClassRelationMapper.updateTeacherClassRelation(teacherClassRelation);
    }

    /**
     * 批量删除老师-班级关系
     *
     * @param ids 需要删除的老师-班级关系主键
     * @return 结果
     */
    @Override
    public int deleteTeacherClassRelationByIds(Long[] ids) {
        return teacherClassRelationMapper.deleteTeacherClassRelationByIds(ids);
    }

    /**
     * 删除老师-班级关系信息
     *
     * @param id 老师-班级关系主键
     * @return 结果
     */
    @Override
    public int deleteTeacherClassRelationById(Long id) {
        return teacherClassRelationMapper.deleteTeacherClassRelationById(id);
    }

    @Override
    @Transactional(rollbackFor = {RuntimeException.class, ServiceException.class})
    public int bindTeacherClass(AddTeacherClassRelationDto form) {
        // step1：删除旧关系
        teacherClassRelationMapper.deleteRelationBySchoolIdAndTeacherId(form.getSchoolId(), form.getTeacherId());
        // step2：绑定对应的数据
        // step3：返回响应结果
        List<InsertTeacherClassRelationDto> insertList = form.getClassId().stream()
                .map(classId -> new InsertTeacherClassRelationDto(form.getSchoolId(), form.getTeacherId(), classId))
                .collect(Collectors.toList());
        return teacherClassRelationMapper.insertTeacherClassRelationBatch(insertList);
    }

    @Override
    public List<TeacherClassInfoVo> queryTeacherBindClassList(Long schoolId, Long teacherId) {
        return teacherClassRelationMapper.selectTeacherClass(schoolId, teacherId, null);
    }

    @Override
    public String bindClassTeacherMain(BindClassTeacherMain form) {
        // step1：检测该学校下面是否存在对应的班级和教师
        Teacher teacher = teacherMapper.selectTeacherByIdAndSchoolId(form.getTeacherId(), form.getSchoolId());
        if (teacher == null) {
            throw new ServiceException("当前学校不存在该教师");
        }
        Clazz clazz = clazzMapper.selectClazzByIdAndSchoolId(form.getClassId(), form.getSchoolId());
        if (clazz == null) {
            throw new ServiceException("当前学校不存在该班级");
        }
        // step2：检测当前教师是否已经绑定了其他班的班主任
        TeacherClassRelation relation = new TeacherClassRelation();
        BeanUtils.copyBeanProp(relation, form);
        TeacherClassRelation isTeacherMain = teacherClassRelationMapper.selectRelationTeacherMain(relation);

        // step3：满足上面的条件则更新关系
        return null;
    }
}
